SystemInstruction

Enum SystemInstruction 

pub enum SystemInstruction {
    CreateAccount {
        lamports: u64,
        space: u64,
        owner: Pubkey,
    },
    Assign {
        owner: Pubkey,
    },
    Transfer {
        lamports: u64,
    },
    CreateAccountWithSeed {
        base: Pubkey,
        seed: String,
        lamports: u64,
        space: u64,
        owner: Pubkey,
    },
    AdvanceNonceAccount {
        authorized: Pubkey,
    },
    WithdrawNonceAccount {
        lamports: u64,
    },
    InitializeNonceAccount {
        authorized: Pubkey,
    },
    AuthorizeNonceAccount {
        authorized: Pubkey,
    },
    Allocate {
        space: u64,
    },
    AllocateWithSeed {
        base: Pubkey,
        seed: String,
        space: u64,
        owner: Pubkey,
    },
    AssignWithSeed {
        base: Pubkey,
        seed: String,
        owner: Pubkey,
    },
    TransferWithSeed {
        lamports: u64,
        seed: String,
        owner: Pubkey,
    },
}
System program instruction types

Variants



CreateAccount

Create a new account 0. [WRITE, SIGNER] Funding account

  1. [WRITE, SIGNER] New account

Fields

lamports: u64

Number of lamports to transfer to the new account

space: u64

Number of bytes of memory to allocate

owner: Pubkey

Address of program that will own the new account



Assign

Assign account to a program 0. [WRITE, SIGNER] Assigned account

Fields

owner: Pubkey

Owner program account



Transfer

Transfer lamports 0. [WRITE, SIGNER] Source account

  1. [WRITE] Destination account

Fields

lamports: u64

Amount of lamports to transfer



CreateAccountWithSeed

Create a new account at an address derived from a base pubkey and a seed 0. [WRITE, SIGNER] Funding account

  1. [WRITE] Created account
  2. [] Base account

Fields

base: Pubkey

Base public key

seed: String

String of ASCII chars, no longer than 32 bytes

lamports: u64

Number of lamports to transfer to the new account

space: u64

Number of bytes of memory to allocate

owner: Pubkey

Address of program that will own the new account



AdvanceNonceAccount

Advance the nonce in a nonce account 0. [WRITE, SIGNER] Nonce account

  1. [] Recent blockhashes sysvar
  2. [SIGNER] Nonce authority

Fields

authorized: Pubkey

Nonce authority



WithdrawNonceAccount

Withdraw funds from a nonce account 0. [WRITE] Nonce account

  1. [WRITE] Recipient account
  2. [SIGNER] Nonce authority
  3. [] Recent blockhashes sysvar

Fields

lamports: u64

Amount of lamports to withdraw



InitializeNonceAccount

Drive state of Nonce account 0. [WRITE] Nonce account

  1. [SIGNER] Nonce authority

Fields

authorized: Pubkey

Nonce authority



AuthorizeNonceAccount

Change the entity authorized to manage nonce 0. [WRITE] Nonce account

  1. [SIGNER] Nonce authority

Fields

authorized: Pubkey

New authority



Allocate

Allocate space in an account 0. [WRITE, SIGNER] Account to allocate

Fields

space: u64

Amount of space to allocate



AllocateWithSeed

Allocate space in an account at an address derived from a base account and a seed 0. [WRITE] Allocated account

  1. [SIGNER] Base account

Fields

base: Pubkey

Base account

seed: String

String of ASCII chars, no longer than 32 bytes

space: u64

Amount of space to allocate

owner: Pubkey

Owner program account



AssignWithSeed

Assign an account at an address derived from a base account and a seed 0. [WRITE] Assigned account

  1. [SIGNER] Base account

Fields

base: Pubkey

Base account

seed: String

String of ASCII chars, no longer than 32 bytes

owner: Pubkey

Owner program account



TransferWithSeed

Transfer lamports from an account at an address derived from a base account and a seed 0. [WRITE] Source account

  1. [WRITE] Destination account
  2. [SIGNER] Base account

Fields

lamports: u64

Amount of lamports to transfer

seed: String

Seed for the source account

owner: Pubkey

Owner program for the seed account

Implementations

Source

impl SystemInstruction

pub fn size(&self) -> usize

The serialized size of the instruction

pub fn serialize(&self) -> Vec<u8>

Serialize the instruction to a byte vector